@@ -53,6 +53,8 @@ int     exit_stat = 0;
 void 
 Usage()
 {
+  fputc('\n', stderr);
+
 #ifdef DEBUG
 
 # ifdef DOS
@@ -70,6 +72,19 @@ Usage()
 # endif				/* DOS */
 
 #endif				/* DEBUG */
+
+    fprintf(stderr, "\n\
+      -c         Write output on stdout, don't remove original.\n\
+      -d         If given, melting is done instead.\n\
+      -g         Use \"greedy\" parsing (1.5% worse, 40% faster).\n\
+      -f         Forces output file to be generated, even if one already\n\
+                 exists, and even if no space is saved by freezeing.\n\
+      -i         Image mode.\n\
+      -t         Text mode.\n\
+      -v         Write freezing statistics. -vv means \"draw progress\n\
+                 indicator\".\n\
+      -V         Write version and compilation options.\n\
+");
 }
 
 void    (*meltfunc) ();		/* To call something for melting */
@@ -113,6 +128,9 @@ long    refers_out = 0, symbols_out = 0;
 long    indc_count, indc_threshold;
 off_t   file_length = 0;	/* initial length of file */
 
+char IOoutbuf[32768];
+char IOinbuf[32768];
+
 RETSIGTYPE      (*bgnd_flag)();
 
 void    writeerr(), copystat(), version(), tune_table();
@@ -178,7 +196,7 @@ void    writeerr(), copystat(), version(), tune_table();
 
 void 
 main(argc, argv)
-register int argc;
+int argc;
 char  **argv;
 {
   short   overwrite = 0;	/* Do not overwrite unless given -f flag */
@@ -204,6 +222,11 @@ char  **argv;
   extern RETSIGTYPE oops();
 #endif
 
+#ifdef __EMX__
+    _response(&argc, &argv);
+    _wildcard(&argc, &argv);
+#endif
+
 #ifndef DOS
   if ((bgnd_flag = signal(SIGINT, SIG_IGN)) != SIG_IGN)
 #endif
@@ -237,6 +260,11 @@ char  **argv;
 #endif
   }
 
+    if (argc == 1 && isatty(0)) {
+      Usage();
+      exit(0);
+    }
+
 #ifdef DOS
 /* use case-insensitive match: parent may not be command.com */
 #ifdef MSDOS
@@ -269,15 +297,18 @@ char  **argv;
 #ifndef DOS
     (void) defopen(deffile);
 #else
+#ifdef TOS
     cp = strrchr(cp, '.');
     *++cp = 'C';
     *++cp = 'N';
     *++cp = 'F';
     *++cp = '\0';
-
-#ifdef TOS
     (void) defopen(argv0);
 #else
+	char config[MAXNAMLEN];
+	_searchenv("freeze.cnf", "INIT", config);
+	if (config[0] == 0)
+	  _searchenv("freeze.cnf", "PATH", config);
     (void) defopen(argv[0]);
 #endif				/* TOS */
 #endif				/* DOS */
@@ -417,11 +448,22 @@ char  **argv;
 	    (void) strcat(tempname, "F");
 	  else {
 	  /* cp now points to file extension */
+#ifdef OS2
+                        (void) strcat(tempname, ".F");
+                        if (!IsFileNameValid(tempname)) {
+			    tail[0] = cp[1];
+			    tail[1] = cp[2];
+			    tail[2] = '\0';
+			    *(++cp) = '\0';
+			    (void) strcat(tempname, "XF");
+                        }
+#else
 	    tail[0] = cp[1];	/* save two chars */
 	    tail[1] = cp[2];
 	    tail[2] = '\0';
 	    *(++cp) = '\0';
 	    (void) strcat(tempname, "XF");
+#endif
 	  }
 	  *fileptr = tempname;
 	}
@@ -449,6 +491,9 @@ char  **argv;
 	  perror(*fileptr);
 	  continue;
 	}
+
+	setvbuf(stdin, IOinbuf, _IOFBF, sizeof(IOinbuf));
+
       /* Check the magic number */
 	if (getchar() != MAGIC1)
 	  goto reject;
@@ -468,6 +513,8 @@ char  **argv;
 	  continue;
 	}
 
+                setvbuf(stdin, IOinbuf, _IOFBF, sizeof(IOinbuf));
+
       /* Generate output filename */
 	precious = 1;
 	(void) strcpy(ofname, *fileptr);
@@ -537,9 +584,17 @@ char  **argv;
 	/* FOOBAR. case */
 	  (void) strcat(ofname, "F");
 	else {
+#ifdef OS2
+			(void) strcat(ofname, ".F");
+			if (!IsFileNameValid(ofname)) {
 	  cp[2] = '\0';
 	  (void) strcat(ofname, "XF");
 	}
+#else
+	  cp[2] = '\0';
+	  (void) strcat(ofname, "XF");
+#endif
+	}
 #else
 	(void) strcat(ofname, ".F");
 #endif				/* DOS */
@@ -604,6 +659,8 @@ char  **argv;
 	  setmode(fileno(stdout), O_BINARY);
 #endif
       }
+            setvbuf(stdout, IOoutbuf, _IOFBF, sizeof(IOoutbuf));
+
     /* Actually do the freezing/melting */
       if (do_melt == 0)
 	freeze();
@@ -771,6 +828,7 @@ char   *ifname;
   else				/* MS-DOS 2.xx bug */
 # endif
 
+    (void) fclose(stdin);
     (void) fclose(stdout);
 
   if (exit_stat == 2 && (!force)) {	/* No freezing: remove file.F */
